﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Analysis;
using HeuristicLab.MainForm;

namespace HeuristicLab.DataPreprocessing.Views {
  [View("Histogram View")]
  [Content(typeof(HistogramContent), true)]
  public partial class HistogramView : PreprocessingChartView {
    public new HistogramContent Content {
      get { return (HistogramContent)base.Content; }
      set { base.Content = value; }
    }

    public HistogramView() {
      InitializeComponent();
      aggregationComboBox.DataSource = Enum.GetValues(typeof(DataRowVisualProperties.DataRowHistogramAggregation));
      aggregationComboBox.SelectedItem = DataRowVisualProperties.DataRowHistogramAggregation.Overlapping;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      classifierComboBox.Items.Clear();
      classifierComboBox.Items.Add("");

      if (Content != null) {
        foreach (string var in GetVariableNamesForGrouping(Content.PreprocessingData)) {
          classifierComboBox.Items.Add(var);
        }

        classifierComboBox.SelectedItem = Content.GroupingVariableName;
      }
    }

    protected override DataTable CreateDataTable(string variableName) {
      var dataTable = new DataTable();

      if (string.IsNullOrEmpty(Content.GroupingVariableName)) {
        var row = Content.CreateDataRow(variableName, DataRowVisualProperties.DataRowChartType.Histogram);
        dataTable.Rows.Add(row);
        return dataTable;
      }

      dataTable.VisualProperties.Title = variableName;

      int variableIndex = Content.PreprocessingData.GetColumnIndex(variableName);
      var variableValues = Content.PreprocessingData.GetValues<double>(variableIndex);
      int groupVariableIndex = Content.PreprocessingData.GetColumnIndex(Content.GroupingVariableName);
      var groupingValues = Enumerable.Empty<string>();

      if (Content.PreprocessingData.VariableHasType<double>(groupVariableIndex)) {
        groupingValues = Content.PreprocessingData.GetValues<double>(groupVariableIndex).Select(x => x.ToString());
      } else if (Content.PreprocessingData.VariableHasType<string>(groupVariableIndex)) {
        groupingValues = Content.PreprocessingData.GetValues<string>(groupVariableIndex);
      } else if (Content.PreprocessingData.VariableHasType<DateTime>(groupVariableIndex)) {
        groupingValues = Content.PreprocessingData.GetValues<DateTime>(groupVariableIndex).Select(x => x.ToString());
      }

      var groups = groupingValues.Zip(variableValues, Tuple.Create).GroupBy(t => t.Item1, t => t.Item2);

      foreach (var group in groups) {
        var classRow = new DataRow();
        classRow.Name = group.Key;
        classRow.VisualProperties.ChartType = DataRowVisualProperties.DataRowChartType.Histogram;
        classRow.VisualProperties.Aggregation = (DataRowVisualProperties.DataRowHistogramAggregation)aggregationComboBox.SelectedItem;
        classRow.Values.AddRange(group);
        dataTable.Rows.Add(classRow);
      }
      return dataTable;
    }


    private void classifierComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      Content.GroupingVariableName = classifierComboBox.SelectedItem.ToString();

      // rebuild datatables
      InitData();
      GenerateLayout();
    }

    private void aggregationComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      foreach (var dt in dataTables.Values) {
        foreach (var row in dt.Rows) {
          row.VisualProperties.Aggregation = (DataRowVisualProperties.DataRowHistogramAggregation)aggregationComboBox.SelectedItem;
        }
      }
    }
  }
}
